"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
function mongooseSubquery(schema, options = {}) {
    const decodeQuery = function () {
        return __awaiter(this, void 0, void 0, function* () {
            const mongooseQuery = this;
            // get reference fields in schema
            const referenceFields = {};
            Object.keys(schema.obj).forEach(fieldKey => {
                const field = schema.obj[fieldKey];
                if (field && field.type && field.type.name === "ObjectId" && field.ref) {
                    referenceFields[fieldKey] = field;
                }
            });
            const decodeRecursive = function (obj) {
                return __awaiter(this, void 0, void 0, function* () {
                    if (Array.isArray(obj)) {
                        obj = Object.assign({}, obj);
                    }
                    yield Promise.all(Object.keys(obj).map((key) => __awaiter(this, void 0, void 0, function* () {
                        const value = obj[key];
                        if (value && typeof value === "object" && value.$subquery && referenceFields[key]) {
                            if (options.transformSubquery) {
                                options.transformSchema && (yield options.transformSchema(schema, mongooseQuery, obj));
                            }
                            // @ts-ignore
                            const referenceModel = mongooseQuery.model.db.model(referenceFields[key].ref);
                            let subquery = referenceModel.find(value.$subquery, "_id");
                            if (options.transformSubquery) {
                                subquery = yield options.transformSubquery(subquery);
                            }
                            const res = yield subquery;
                            const resIds = res.map(doc => doc.id);
                            delete value.$subquery;
                            value.$in = resIds;
                        }
                        else if (value && typeof value === "object") {
                            yield decodeRecursive(value);
                        }
                    })));
                });
            };
            mongooseQuery.getQuery && (yield decodeRecursive(mongooseQuery.getQuery()));
        });
    };
    schema.pre('count', decodeQuery);
    schema.pre('deleteMany', decodeQuery);
    schema.pre('deleteOne', decodeQuery);
    schema.pre('find', decodeQuery);
    schema.pre('findOne', decodeQuery);
    schema.pre('findOneAndDelete', decodeQuery);
    schema.pre('findOneAndRemove', decodeQuery);
    schema.pre('findOneAndUpdate', decodeQuery);
    schema.pre('remove', decodeQuery);
    schema.pre('update', decodeQuery);
    schema.pre('updateOne', decodeQuery);
    schema.pre('updateMany', decodeQuery);
}
exports.default = mongooseSubquery;
//# sourceMappingURL=index.js.map